%basins_stablemanifold.m, by Daniel Lyons & Sara Wang (March 2014):
%Purpose: Computes the 2D stable manifold of the origin and triangulates
%the surface.
%Lambda Range: lambda>1.3333
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; close all; clc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%filepath (needs to be changed locally to folder containing "CCFM_manifolds")
mypath = '/Users/danielclyons/Dropbox/CCFM_large_folder';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath(strcat(mypath,'/CCFM_manifolds/functions/'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%initialize parameters
global c lambda;
c = 3.0;
lambda = 1.4;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%solution tolerance (f_solve)
sol_tol = 1.0e-8;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%set viewpoint for 3D plots
azimuth = 52; elevation = 10;

%numerical perturbation options
delta_x = 0.01;%0.1
step = 0.004;
ode_system = 'odeccfm';
ode_system_fsolve = 'fluxgate';

%color_hash for plotting
color_hash = {'k.','go','ko'};
%{stable, saddle (2D unstable), saddle (2D stable)}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%obtain fixed points
fixed_points = [
    0 0 0;
    -1 -1 -1;
    1 1 1;
];
fixed_points_array = compute_fixed_points(ode_system_fsolve,fixed_points,sol_tol);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%compute invariant manifold via numerical integration
ti = 0.0; h = -0.25; tf = -60;%integration time data
time = ti:h:tf; %set time array
save_time_start_index = find(time==-0.25);%start saving at time(save_time_start_index)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%compute jacobian and stability info for origin
for ii=1:length(fixed_points_array)
    fpa_jac = CCFM_Jac(fixed_points_array(ii,:));
    
    %evals = eig(fpa_jac)
    [type,eig_val]  = stability_info(fpa_jac);
    
    %0: no information
    %1: (sink: 3D stable manifold (3 eigenvalues with negative real part)
    %2: (source: 3D unstable manifold (3 eigenvalues with positive real part)
    %3: (saddle: 2D stable manifold (2 eigenvalues with negative real part)
    %4: (saddle: 2D unstable manifold (2 eigenvalues with positive real part)
    if type == 0
        verb_state = 'unknown';
    elseif type == 1
        verb_state = 'sink (3D stable)';
    elseif type == 2
        verb_state = 'source (3D unstable)';
    elseif type == 3
        verb_state = 'saddle (2D stable)';
    elseif type == 4
        verb_state = 'saddle (2D unstable)';
    end

    %print out stability info
    disp(['(',num2str(fixed_points_array(ii,1)),',',num2str(fixed_points_array(ii,2)),',',num2str(fixed_points_array(ii,3)),'): ',verb_state]);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%compute 2D stable manifold of origin
%circle parameters
points = 201;
radius = 0.05;%3e-2;
my_face_alpha = 0.5;

fixed_points = fixed_points_array(1,:);

%compute eigenvalues for 2D stable manifolds with integrating backwards
%in time beginning with a circle of initial conditions with radius "r"
%and "points" points in the 2D space spanned by the two eigenvectors
%at the equilibrium "fixed_points" with exactly 2 eigenvalues with
%negative real part
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%compute Jacobian, eigenvalues, eigenvector at saddle points with 2D stable
%manifolds
x0 = fixed_points;
DF = CCFM_Jac(x0);
[eigvec,D] = eig(DF);
for ee=1:3
    eigval(ee) = D(ee,ee);
end
%obtain the 2 eigenvectors of 2D linear eigenspace
eigvec_stable = eigvec(:,eigval<0);
%since the 2 stable eigenvectors are complex conjugates
%we take the real and imaginary parts as the two eigenvectors
eigvec_stable = [real(eigvec_stable(:,1)) imag(eigvec_stable(:,1))];

%obtain coordinates of circle with radius r in (x1,x2) plane
[x1,x2] = planar_circle(points,radius);

%perform linear mapping of unit circle onto plane in R3 spanned by W1,W2
[x1_new,x2_new,x3_new] = curve_map_R2_R3(x1,x2,eigvec_stable(:,1),eigvec_stable(:,2),points);
%translate mapping to saddle point
x1_new = x1_new + x0(1);
x2_new = x2_new + x0(2);
x3_new = x3_new + x0(3);
x_new = [x1_new' x2_new' x3_new'];
remove_small_norm_points = 0;
[x,y,z] = manifold_compute_circle(fixed_points,time,save_time_start_index,delta_x,step,ode_system,x_new,remove_small_norm_points);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % filter out extra points within small tolerance of each other
norm_tol = 0.01;
[x,y,z] = redundant_point_filter(x,y,z,norm_tol);

%x(1) = []; y(1) = []; z(1) = [];%remove origin if necessary f/
%triangulation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%invariant manifold surface plot showing faces and edges
FP = fixed_points_array(2:3,:);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot invariant manifold points only
figure;
point_plot(x,y,z);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot surfaces with Delaunay2_5D program
figure;
Delaunay_special_plot(x,y,z,1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot sample trajectories
num_traj = 2; %number of trajectories
min = -0.5; max = 0.5;
delta = (max - min)/(num_traj - 1);
mygrid = min:delta:max; %mygrid = -1.0:0.5:1.0;
%FP_1 = [-1.000 -1.000 -1.000];
%FP_2 = -FP_1;
%fixed_points_array = zeros(2,3);
%fixed_points_array(1,:) = FP_1;
%fixed_points_array(2,:) = FP_2;

%plot_trajectories(mygrid,FP,lambda);
scale = 1.5;
axis([-scale scale -scale scale -scale scale]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%plot stable fixed points
hold on;
plot3(FP(1,1),FP(1,2),FP(1,3),'k.','markersize',18); hold on;
plot3(FP(2,1),FP(2,2),FP(2,3),'k.','markersize',18); hold on;
plot3(0,0,0,'bo','markersize',9);

