%CCFM_equilibria_lambda_color.m:
%Purpose: Performs CCFM equilibria continuation and produces a 3D plot with
%color darkness indicating the value of the parameter lambda.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; close all; clc;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%initialize parameters
global c lambda;
c = 3.0;
lambda = 0; %initial lambda value
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%solution tolerance (f_solve)
sol_tol = 1.0e-8;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%set viewpoint for 3D plots
azimuth = 120;
elevation = 28;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%input functions
ode_system_fsolve = 'fluxgate';
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%approximate equilibria at lambda=0
fixed_points_array = [

%%%%%%%%%%%% These 12 points exist over -0.4475<lambda<0.4349
%1D stable (saddle)
1 0 0;
-1 0 0;
0 1 0;
0 -1 0;
0 0 1;
0 0 -1;

%2D stable (saddle)
0 1 1;
1 0 1;
1 1 0;
0 -1 -1;
-1 0 -1;
-1 -1 0;

%%%%%%%%%%%% These 12 points exist over -0.4475<lambda<0.4349
0 -1 1;
0 1 -1;
-1 0 1;
1 0 -1;
-1 1 0;
1 -1 0;

%sinks (async)
1 1 -1;
1 -1 1;
1 -1 -1;
-1 1 1;
-1 1 -1;
-1 -1 1;

%%%%%%%%%%%% These 2 points exist over -0.6667<lambda
%sinks (sync)
1 1 1;
-1 -1 -1;
%%%%%%%%%%%% This point exists for all real lambda
%3D unstable
0 0 0;
];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%compute equilibria at lambda=0 with high accuracy
final_fixed_point_array = compute_fixed_points(ode_system_fsolve,fixed_points_array,sol_tol);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plotting info
%1('k-'): (sink: 3D stable manifold (3 eigenvalues with negative real part)
%2:('k--') (source: 3D unstable manifold (3 eigenvalues with positive real part)
%3:('b--') (saddle: 2D stable manifold (2 eigenvalues with negative real part)
%4:('r--') (saddle: 2D unstable manifold (2 eigenvalues with positive real part)
%color_hash={1,2,3,4}
color_hash = {'k.','ko','bo','ro'};
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
figure;
type_array = zeros(27,1);
%compute jacobian and stability info
for ii = 1:length(final_fixed_point_array)
    fpa_jac = CCFM_Jac(final_fixed_point_array(ii,:));
    
    %evals = eig(fpa_jac)
    [type,eig_val]  = stability_info(fpa_jac);
    
    type_array(ii) = type;
    %0: no information
    %1: (sink: 3D stable manifold (3 eigenvalues with negative real part)
    %2: (source: 3D unstable manifold (3 eigenvalues with positive real part)
    %3: (saddle: 2D stable manifold (2 eigenvalues with negative real part)
    %4: (saddle: 2D unstable manifold (2 eigenvalues with positive real part)
    if type == 0
        verb_state = 'unknown';
    elseif type == 1
        verb_state = 'sink (3D stable)';
    elseif type == 2
        verb_state = 'source (3D unstable)';
    elseif type == 3
        verb_state = 'saddle (2D stable)';
    elseif type == 4
        verb_state = 'saddle (2D unstable)';
    end

    
    %print out stability info
    disp(['(',num2str(final_fixed_point_array(ii,1)),',',num2str(final_fixed_point_array(ii,2)),',',num2str(final_fixed_point_array(ii,3)),'): ',verb_state]);
    
    hold on;
    plot3(final_fixed_point_array(ii,1),final_fixed_point_array(ii,2),final_fixed_point_array(ii,3),color_hash{type},'markersize',9);
end
title('Equilibria at \lambda=0','fontsize',18);
view([azimuth elevation]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%left branch: equilibria continuation to from lambda=0 to left saddle nodes
%lambda=-0.4345 and -0.4475
points = 101;
fp_left = zeros(27,3*points);
fp_left(:,3*points-2:3*points) = final_fixed_point_array;

lambda_min1 = -0.447; %lambda_max1 = 0.4344;
lambda_min2 = -0.433; %lambda_max2 = 0.4348;
lambda_min3 = -0.667;    %lambda_max3 = 0.5;

delta_lambda_1 = (0-lambda_min1)/(points-1);
delta_lambda_2 = (0-lambda_min2)/(points-1);
delta_lambda_3 = (0-lambda_min3)/(points-1);

lambda_range_1 = lambda_min1:delta_lambda_1:0;
lambda_range_2 = lambda_min2:delta_lambda_2:0;
lambda_range_3 = lambda_min3:delta_lambda_3:0;


fp_points1 = [1:6 7:12];
fp_points2 = [13:18 19:24];
fp_points3 = [25:27];

for jj=1:3

    jj
    
    if jj==1
        lambda_range = lambda_range_1;
        fp_points = fp_points1;
    elseif jj==2
        lambda_range = lambda_range_2;
        fp_points = fp_points2;
    elseif jj==3
        lambda_range = lambda_range_3;
        fp_points = fp_points3;
    end
    
    fixed_points_array = fp_left(fp_points,3*points-2:3*points);
    
    %perform continuation for left branch
    for ii=points-1:-1:1
        lambda = lambda_range(ii);
        fixed_points_array = compute_fixed_points(ode_system_fsolve,fixed_points_array,sol_tol);
        fp_left(fp_points,3*ii-2:3*ii) = fixed_points_array;
    end

        

end

%save lambda arrays
lambda_array1 = lambda_range_1;
lambda_array2 = lambda_range_2;
lambda_array3 = lambda_range_3;


figure;
x_range = 1:3:3*points-2;
y_range = 2:3:3*points-1;
z_range = 3:3:3*points;

m_size = 9;

for ii=1:27
    equil_index = ii;
    hold on; plot3(fp_left(equil_index,x_range),...
        fp_left(equil_index,y_range),fp_left(equil_index,z_range),...
        color_hash{type_array(equil_index)},'markersize',m_size);
end
title('Left Branch','fontsize',18);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%right branch: equilibria continuation from lambda=0 to right saddle nodes
%lambda=0.4346 and 0.4349
%points = 31;
fp_right = zeros(27,3*points);
fp_right(:,1:3) = final_fixed_point_array;

lambda_min1 = 0; lambda_max1 = 0.4348;
lambda_min2 = 0; lambda_max2 = 0.4344;
lambda_min3 = 0; lambda_max3 = 0.4348;

delta_lambda_1 = (lambda_max1 - lambda_min1)/(points-1);
delta_lambda_2 = (lambda_max2 - lambda_min2)/(points-1);
delta_lambda_3 = (lambda_max3 - lambda_min3)/(points-1);

lambda_range_1 = lambda_min1:delta_lambda_1:lambda_max1;
lambda_range_2 = lambda_min2:delta_lambda_2:lambda_max2;
lambda_range_3 = lambda_min3:delta_lambda_3:lambda_max3;


fp_points1 = [1:6 13:18];
fp_points2 = [7:12 19:24];
fp_points3 = [25:27];

for jj=1:3

    jj
    
    if jj==1
        lambda_range = lambda_range_1;
        fp_points = fp_points1;
    elseif jj==2
        lambda_range = lambda_range_2;
        fp_points = fp_points2;
    elseif jj==3
        lambda_range = lambda_range_3;
        fp_points = fp_points3;
    end
    
    fixed_points_array = fp_right(fp_points,1:3);
    
    %perform continuation for left branch
    for ii=2:points
        lambda = lambda_range(ii);
        fixed_points_array = compute_fixed_points(ode_system_fsolve,fixed_points_array,sol_tol);
        fp_right(fp_points,3*ii-2:3*ii) = fixed_points_array;
    end


        

end

%save lambda arrays
lambda_array1 = [lambda_array1 lambda_range_1(2:end)];
lambda_array2 = [lambda_array2 lambda_range_2(2:end)];
lambda_array3 = [lambda_array3 lambda_range_3(2:end)];


figure;
x_range = 1:3:3*points-2;
y_range = 2:3:3*points-1;
z_range = 3:3:3*points;

m_size = 9;

for ii=1:27
    equil_index = ii;
    hold on; plot3(fp_right(equil_index,x_range),...
        fp_right(equil_index,y_range),fp_right(equil_index,z_range),...
        color_hash{type_array(equil_index)},'markersize',m_size);
end
title('Right Branch','fontsize',18);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot left and right branches together
fp_range = horzcat(fp_left,fp_right(:,4:end));
x_range = 1:3:3*(2*points-1)-2;
y_range = 2:3:3*(2*points-1)-1;
z_range = 3:3:3*(2*points-1);

m_size = 9;

figure;
for ii=1:27
    equil_index = ii;
    hold on; plot3(fp_range(equil_index,x_range),...
        fp_range(equil_index,y_range),fp_range(equil_index,z_range),...
        color_hash{type_array(equil_index)},'markersize',m_size);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot two branches with darkness according to lambda value
marker_hash = {'.', 'o', 'o', 'o'};
delta_c = (1-0.5)/((2*points-1)-1);
%c_range = 0.5:delta_c:1;
c_range = 1:-delta_c:0.5;%black and dark blue match up

m_size = 50;

%c_range_black = 0.5:-delta_c:0;
c_range_black = 0.5:-delta_c:0;


figure;
for ii=1:27
    
    C = zeros(2*points-1,3);
    
    if type_array(ii)==1
        C(:,1) = c_range_black;
        C(:,2) = c_range_black;
        C(:,3) = c_range_black;
        hold on;
        scatter3(fp_range(ii,x_range),fp_range(ii,y_range),...
            fp_range(ii,z_range),4*m_size,C,marker_hash{1});
    elseif type_array(ii)==2
        
        C(:,1) = c_range_black;
        C(:,2) = c_range_black;
        C(:,3) = c_range_black;
        hold on;
        scatter3(fp_range(ii,x_range),fp_range(ii,y_range),...
            fp_range(ii,z_range),m_size,C,marker_hash{2});
    elseif type_array(ii)==3
        C(:,3) = c_range;
        hold on;
        scatter3(fp_range(ii,x_range),fp_range(ii,y_range),...
            fp_range(ii,z_range),m_size,C,marker_hash{3});
    elseif type_array(ii)==4
        C(:,1) = c_range;
        hold on;
        scatter3(fp_range(ii,x_range),fp_range(ii,y_range),...
            fp_range(ii,z_range),m_size,C,marker_hash{4});
    end
end

axis([-1 1 -1 1 -1 1]);

xlabel('x_1','fontsize',18);
ylabel('x_2','fontsize',18);
zlabel('x_3','fontsize',18);
set(gca,'fontsize',18); grid on;
view([azimuth elevation]);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plot a few equilbria

equil_to_plot = [1 8 16 20 25];
figure;
ii = 1;
for jj=1:length(equil_to_plot)    
        ii = equil_to_plot(jj);
        C = zeros(2*points-1,3);

        if type_array(ii)==1
            
            C(:,1) = c_range_black;
            C(:,2) = c_range_black;
            C(:,3) = c_range_black;
            hold on;
            scatter3(fp_range(ii,x_range),fp_range(ii,y_range),...
                fp_range(ii,z_range),4*m_size,C,marker_hash{1});
        elseif type_array(ii)==2

            C(:,1) = c_range_black;
            C(:,2) = c_range_black;
            C(:,3) = c_range_black;
            hold on;
            scatter3(fp_range(ii,x_range),fp_range(ii,y_range),...
                fp_range(ii,z_range),m_size,C,marker_hash{2});
        elseif type_array(ii)==3
            C(:,3) = c_range;
            hold on;
            scatter3(fp_range(ii,x_range),fp_range(ii,y_range),...
                fp_range(ii,z_range),m_size,C,marker_hash{3});
        elseif type_array(ii)==4
            C(:,1) = c_range;
            hold on;
            scatter3(fp_range(ii,x_range),fp_range(ii,y_range),...
                fp_range(ii,z_range),m_size,C,marker_hash{4});
        end
end
axis([-1 1 -1 1 -1 1]);

xlabel('x_1','fontsize',18);
ylabel('x_2','fontsize',18);
zlabel('x_3','fontsize',18);
set(gca,'fontsize',18); grid on;
view([azimuth elevation]);

hold on;plot3(0,0,0,'ko','markersize',9); %plot origin
